//+------------------------------------------------------------------+
//|                                                 SignalADX-MA.mqh |
//|                      Copyright  2010, MetaQuotes Software Corp. |
//|                                        http://www.metaquotes.net |
//|                                              Revision 2010.09.03 |
//+------------------------------------------------------------------+
#include <Expert\ExpertSignal.mqh>
//--- inputs
input int    Inp_Signal_ADX_MA_Period_ADX =8;    // Signal::ADX_MA::Period_ADX
input double Inp_Signal_ADX_MA_Minimum_ADX=22.0; // Signal::ADX_MA::Minimum_ADX
input int    Inp_Signal_ADX_MA_Period_MA  =8;    // Signal::ADX_MA::Period_MA
input int    Inp_Signal_ADX_MA_StopLoss   =30;   // Signal::ADX_MA::StopLoss
input int    Inp_Signal_ADX_MA_TakeProfit =100;  // Signal::ADX_MA::TakeProfit
//+------------------------------------------------------------------+
//| Class CSignalADX_MA.                                             |
//| Appointment: Class trading signals cross price and MA with ADX.  |
//|              Derives from class CExpertSignal.                   |
//+------------------------------------------------------------------+
class CSignalADX_MA : public CExpertSignal
  {
protected:
   CiADX            *m_ADX;
   CiMA             *m_MA;
   CiClose          *m_close;

public:
                     CSignalADX_MA();
   //---
   virtual bool      InitIndicators(CIndicators *indicators);
   //---
   virtual bool      CheckOpenLong(double& price,double& sl,double& tp,datetime& expiration);
   virtual bool      CheckCloseLong(double& price);
   virtual bool      CheckOpenShort(double& price,double& sl,double& tp,datetime& expiration);
   virtual bool      CheckCloseShort(double& price);

protected:
   bool              InitADX(CIndicators *indicators);
   bool              InitMA(CIndicators *indicators);
   bool              InitClose(CIndicators *indicators);
   //---
   double            PlusADX(int ind)    { return(m_ADX.Plus(ind));            }
   double            MainADX(int ind)    { return(m_ADX.Main(ind));            }
   double            MinusADX(int ind)   { return(m_ADX.Minus(ind));           }
   double            MA(int ind)         { return(m_MA.Main(ind));             }
   double            Close(int ind)      { return(m_close.GetData(ind));       }
   double            StateADX(int ind)   { return(PlusADX(ind)-MinusADX(ind)); }
   double            StateMA(int ind)    { return(MA(ind)-MA(ind+1));          }
   double            StateClose(int ind) { return(Close(ind)-MA(ind));         }
  };
//+------------------------------------------------------------------+
//| Constructor CSignalADX_MA.                                       |
//| INPUT:  no.                                                      |
//| OUTPUT: no.                                                      |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
void CSignalADX_MA::CSignalADX_MA()
  {
//--- initialize protected data
   m_MA   =NULL;
   m_ADX  =NULL;
   m_close=NULL;
  }
//+------------------------------------------------------------------+
//| Create indicators.                                               |
//| INPUT:  indicators -pointer of indicator collection.             |
//| OUTPUT: true-if successful, false otherwise.                     |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
bool CSignalADX_MA::InitIndicators(CIndicators *indicators)
  {
//--- create and initialize ADX indicator
   if(!InitADX(indicators))   return(false);
//--- create and initialize MA indicator
   if(!InitMA(indicators))    return(false);
//--- create and initialize close indicator
   if(!InitClose(indicators)) return(false);
//--- ok
   return(true);
  }
//+------------------------------------------------------------------+
//| Create Open indicators.                                          |
//| INPUT:  indicators -pointer of indicator collection.             |
//| OUTPUT: true-if successful, false otherwise.                     |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
bool CSignalADX_MA::InitADX(CIndicators *indicators)
  {
   string symbol=m_symbol.Name();
//--- create open indicator and add it to collection
   if(m_ADX==NULL)
      if((m_ADX=new CiADX)==NULL)
         return(false);
   if(!indicators.Add(m_ADX))
      return(false);
//--- initialize open indicator
   if(!m_ADX.Create(symbol,m_period,Inp_Signal_ADX_MA_Period_ADX))
      return(false);
//--- ok
   return(true);
  }
//+------------------------------------------------------------------+
//| Create MA indicators.                                            |
//| INPUT:  indicators -pointer of indicator collection.             |
//| OUTPUT: true-if successful, false otherwise.                     |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
bool CSignalADX_MA::InitMA(CIndicators *indicators)
  {
   string symbol=m_symbol.Name();
//--- create MA indicator and add it to collection
   if(m_MA==NULL)
      if((m_MA=new CiMA)==NULL)
         return(false);
   if(!indicators.Add(m_MA))
      return(false);
//--- initialize EMA indicator
   if(!m_MA.Create(symbol,m_period,Inp_Signal_ADX_MA_Period_MA,0,MODE_EMA,PRICE_CLOSE))
      return(false);
//--- ok
   return(true);
  }
//+------------------------------------------------------------------+
//| Create Close indicators.                                         |
//| INPUT:  indicators -pointer of indicator collection.             |
//| OUTPUT: true-if successful, false otherwise.                     |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
bool CSignalADX_MA::InitClose(CIndicators *indicators)
  {
   string symbol=m_symbol.Name();
//--- create close indicator and add it to collection
   if(m_close==NULL)
      if((m_close=new CiClose)==NULL)
         return(false);
   if(!indicators.Add(m_close))
      return(false);
//--- initialize close indicator
   if(!m_close.Create(symbol,m_period))
      return(false);
//--- ok
   return(true);
  }
//+------------------------------------------------------------------+
//| Check conditions for long position open.                         |
//| INPUT:  price      - refernce for price,                         |
//|         sl         - refernce for stop loss,                     |
//|         tp         - refernce for take profit,                   |
//|         expiration - refernce for expiration.                    |
//| OUTPUT: true-if condition performed, false otherwise.            |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
bool CSignalADX_MA::CheckOpenLong(double& price,double& sl,double& tp,datetime& expiration)
  {
   bool Buy_Condition_1=(StateMA(0)>0 && StateMA(1)>0);             // MA-8 
   bool Buy_Condition_2=(StateClose(1)>0);                          //       MA-8
   bool Buy_Condition_3=(MainADX(0)>Inp_Signal_ADX_MA_Minimum_ADX); //   ADX ,   (22)
   bool Buy_Condition_4=(StateADX(0)>0);                            // +DI ,  -DI
//---
   price=0.0;
   sl   =m_symbol.Ask()-Inp_Signal_ADX_MA_StopLoss*m_adjusted_point;
   tp   =m_symbol.Ask()+Inp_Signal_ADX_MA_TakeProfit*m_adjusted_point;
//---
   return(Buy_Condition_1 && Buy_Condition_2 && Buy_Condition_3 && Buy_Condition_4);
  }
//+------------------------------------------------------------------+
//| Check conditions for long position close.                        |
//| INPUT:  price - refernce for price.                              |
//| OUTPUT: true-if condition performed, false otherwise.            |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
bool CSignalADX_MA::CheckCloseLong(double& price)
  {
   bool Sell_Condition_1=(StateMA(0)<0 && StateMA(1)<0);             // MA-8 
   bool Sell_Condition_2=(StateClose(1)<0);                          //     MA-8
   bool Sell_Condition_3=(MainADX(0)>Inp_Signal_ADX_MA_Minimum_ADX); //   ADX value   (22)
   bool Sell_Condition_4=(StateADX(0)<0);                            // -DI ,  +DI
//---
   price=0.0;
//---
   return(Sell_Condition_1 && Sell_Condition_2 && Sell_Condition_3 && Sell_Condition_4);
  }
//+------------------------------------------------------------------+
//| Check conditions for short position open.                        |
//| INPUT:  price      - refernce for price,                         |
//|         sl         - refernce for stop loss,                     |
//|         tp         - refernce for take profit,                   |
//|         expiration - refernce for expiration.                    |
//| OUTPUT: true-if condition performed, false otherwise.            |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
bool CSignalADX_MA::CheckOpenShort(double& price,double& sl,double& tp,datetime& expiration)
  {
   bool Sell_Condition_1=(StateMA(0)<0 && StateMA(1)<0);             // MA-8 
   bool Sell_Condition_2=(StateClose(1)<0);                          //     MA-8
   bool Sell_Condition_3=(MainADX(0)>Inp_Signal_ADX_MA_Minimum_ADX); //   ADX value   (22)
   bool Sell_Condition_4=(StateADX(0)<0);                            // -DI ,  +DI
//---
   price=0.0;
   sl   =m_symbol.Bid()+Inp_Signal_ADX_MA_StopLoss*m_adjusted_point;
   tp   =m_symbol.Bid()-Inp_Signal_ADX_MA_TakeProfit*m_adjusted_point;
//---
   return(Sell_Condition_1 && Sell_Condition_2 && Sell_Condition_3 && Sell_Condition_4);
  }
//+------------------------------------------------------------------+
//| Check conditions for short position close.                       |
//| INPUT:  price - refernce for price.                              |
//| OUTPUT: true-if condition performed, false otherwise.            |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
bool CSignalADX_MA::CheckCloseShort(double& price)
  {
   bool Buy_Condition_1=(StateMA(0)>0 && StateMA(1)>0);             // MA-8 
   bool Buy_Condition_2=(StateClose(1)>0);                          //       MA-8
   bool Buy_Condition_3=(MainADX(0)>Inp_Signal_ADX_MA_Minimum_ADX); //   ADX ,   (22)
   bool Buy_Condition_4=(StateADX(0)>0);                            // +DI ,  -DI
//---
   price=0.0;
//---
   return(Buy_Condition_1 && Buy_Condition_2 && Buy_Condition_3 && Buy_Condition_4);
  }
//+------------------------------------------------------------------+
